/*
 * poli.c
 */

#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include "poli.h"

/* Funo auxiliar */
ap_poli cria_no(float coef, int expo, ap_poli prox) {
  if (coef != 0) {
    ap_poli n = (ap_poli) malloc (sizeof (struct termo));
    n->coef = coef;
    n->expo = expo;
    n->prox = prox;
    return n;
  } else
    return prox;
}

/* Cria um polinomio formado por um nico termo de coeficiente coef
   e expoente expo. */
ap_poli cria_termo(float coef, int expo) {
  return cria_no(coef, expo, NULL);
}


/* Soma os polinmios apontados por p e q, sem destru-los e retorna
   o resultado. */
ap_poli soma(ap_poli p, ap_poli q) {
  if (p == NULL)
    return copia(q);
  if (q == NULL)
    return copia(p);
  if (p->expo > q->expo)
    return cria_no(p->coef, p->expo, 
		   soma(p->prox, q));
  if (p->expo == q->expo)
    return cria_no(p->coef+q->coef, p->expo, 
		   soma(p->prox, q->prox));
  return cria_no(q->coef, q->expo, 
		 soma(p, q->prox));
}

/* Multiplica o polinmio p por um escalar, sem destru-lo, e retorna o
   polinmio resultante. */
ap_poli mult_escalar(ap_poli p, float escalar) {
  if (p == NULL || escalar == 0)
    return NULL;
  return cria_no(p->coef * escalar, 
		 p->expo,
		 mult_escalar(p->prox, escalar));
}

/* Subtrai q de p e retorna o polinmio resultante. As funes 
   multEscalar e soma podem ser utilizadas para fazer a subtrao. */
ap_poli subtrai(ap_poli p, ap_poli q) {
  return NULL;
}

ap_poli mult_termo(ap_poli p, float coef, int expo) {
  return NULL;
}

/* Multiplica os polinmios apontados por p e q, sem destru-los, e 
   retorna o resultado. Os ns intermedirios, criados durante o
   processo, devem ser destrudos. */
ap_poli multiplica(ap_poli p, ap_poli q) {
  return NULL;
}
 
/* Retorna uma cpia do polinmio p. A funo multEscalar pode
   ser utilizada para fazer a cpia. */
ap_poli copia(ap_poli p) {
  if (p == NULL)
    return NULL;
  return cria_no(p->coef, p->expo, copia(p->prox));
}

/* Desaloca a memria correspondente ao polinmio p. */
void libera(ap_poli p) {
  if (p != NULL) {
    libera(p->prox);
    free(p);
  }
}

/* Escreve o polinmio apontado por p */
void escreve(ap_poli p) {

  if (p == NULL)
    printf("0");
  else { 
    /* Escreve o primeiro termo */
    printf ("%.1f", p->coef);
    if (p->expo > 1)
      printf ("x^%d", p->expo);
    else if (p->expo == 1)
      printf ("x");
    p = p->prox;
    /* Escreve os outros termos */
    while (p != NULL) {
      if (p->coef >= 0) 
	printf (" + %.1f", p->coef);
      else
	printf (" - %.1f", - p->coef);
      if (p->expo > 1)
	printf ("x^%d", p->expo);
      else if (p->expo == 1)
	printf ("x");
      p = p->prox;
    }
  }
  printf ("\n");
}


void teste() {
  /* Cria 3x^2 + 5x - 1 */
  ap_poli p = cria_no (3, 2, cria_no(5, 1, cria_no(-1, 0, NULL)));
  escreve(p);
  /* Cria 4x^3 - 5x */
  ap_poli q = cria_no (4, 3, cria_no(-5, 1, NULL));
  escreve(q);
  ap_poli r = soma(p, q);
  escreve(r);

  ap_poli m = mult_escalar(p, 3);
  escreve(m);

  ap_poli m2 = multiplica(p, q);
  escreve(m2);

  libera(p);
  libera(q);
  libera(r);
  libera(m);
  libera(m2);

}

int main() {
 struct mallinfo info;
 int mall_minblks;
  
  /* Utiliza o sistema de memria dinmica uma vez, alocando e
     desalocando um n, e verifica o nmero mnimo de blocos na
     memria. */
  ap_poli p = (ap_poli) malloc (sizeof (struct termo));
  free(p);
  info = mallinfo();
  mall_minblks = info.uordblks;

  teste();

  /* Verifica a utilizao de memria dinmica */
  {
    struct mallinfo info;
    info = mallinfo();
    if (info.uordblks > mall_minblks)
      printf("Memria sobrando...\n");
  }
  return 0;

}
